/*

Copyright 2010 James A. Mason, York University, Toronto, Canada
http://www.yorku.ca/jmason

Unless required by applicable law or agreed to in writing, this software
is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
ANY KIND, either express or implied.

You may use and modify it for research purposes, provided that
the copyright holder above is acknowledged as the original author.

 */

package cards1;

import asd.*;

import java.util.*;

/*
   Cardgram1Semantics defines the operations of setting
   and testing values of semantic features that are specified
   in the semantic action and semantic value augmentations
   of nodes in the ASD grammar Cardgram1.grm .
*/
public class Cardgram1Semantics
{
   public Cardgram1Semantics()
   {
   }

   public void setParser(ASDParser p)
   {
      parser = p;
   }

// Following are functions corresponding to and invoked by grammar nodes
// in Cardgram1.grm, along with some helping functions:

   public HashMap features() { return (HashMap) parser.features(); }

   public Object nodeValue() { return parser.currentNode().value(); }

   public void raiseFeatures() { parser.raiseFeatures(); }

   public void raiseFeaturesChecking() { parser.raiseFeaturesChecking(); }

   public void set(String feature, Object value) { parser.set(feature, value); }

   public Object valueOf(String given) { return parser.valueOf(given); }

   public String cardgram_$$_3()
   {
      String command = (String) valueOf("command");
      String direction = (String) valueOf("direction");
      if (command.equals("spread"))
      {
         if (direction != null && ! direction.equals("out"))
            return parser.NOADVANCE;
      }
      else if (command.equals("shuffle") || command.equals("stack"))
      {
         if ( ! (direction == null || direction.equals("up")) )
            return parser.NOADVANCE;
      }
      else if (command.equals("turn"))
      {
         if (direction == null)
         {  String orientation = (String) valueOf("orientation");
            if (orientation == null)
               set("direction", "over");
            else
               set("direction", orientation); // from "face down" or "face up"
         }
         else if ( ! (direction.equals("down") || direction.equals("over")
            || direction.equals("up") ) )
            return parser.NOADVANCE;
      }

      return null;
   }

   public Object cardgram_$$_4_v()
   {
      return valueOf("clauses");
   }

   public String cardgram_ADVDIR_1()
   {
      set("direction", nodeValue());
      return null;
   }

   public String cardgram_ADVDIR_2()
   {
      String direction = (String) valueOf("direction");
      if (direction != null &&
          ! direction.equals((String) nodeValue()))
         return parser.NOADVANCE;
      set("direction", nodeValue());
      return null;
   }

   public String cardgram_all_1()
   {
      set("number", "plural");
      set("quantity", "several");
      set("inclusivity", "all");
      return null;
   }

   public String cardgram_all_2()
   {
      set("inclusivity", "all");
      return null;
   }

   public String cardgram_down_2()
   {
      set("orientation", "down");
      return null;
   }

   public String cardgram_each_1()
   {
      set("aggregation", "each");
      set("inclusivity", "all");
      set("number", "singular");
      set("quantity", "several");
      return null;
   }

   public String cardgram_every_1()
   {
      set("aggregation", "each");
      set("inclusivity", "all");
      set("number", "singular");
      set("quantity", "several");
      return null;
   }

   public String cardgram_here_1()
   {
      set("where", "here");
      return null;
   }

   public String cardgram_it_1()
   {
      set("which", "anaphoric");
      set("quantity", "one");
      set("number", "singular");
      return null;
   }

   public String cardgram_MAINCLAUSE_1()
   {
      ArrayList<HashMap> clauses
         = new ArrayList<HashMap>(2);
      Object nodeVal = nodeValue();
      if (nodeVal instanceof HashMap)
      {
         clauses.add((HashMap) nodeVal);
         set("clauses", clauses);
         return null;
      }
      else
         return parser.NOADVANCE;
   }

   public String cardgram_MAINCLAUSE_2()
   {
      Object nodeVal = nodeValue();
      Object clauseVal = valueOf("clauses");
      if (clauseVal instanceof ArrayList
          && nodeVal instanceof HashMap)
      {
         ((ArrayList) clauseVal).add(nodeVal);
         return null;
      }
      else
         return parser.NOADVANCE;
   }

   public Object cardgram_MAINCLAUSE_2_v()
   {
      return valueOf("clauses");
   }

   public String cardgram_many_1()
   {
      set("command", "count");
      return null;
   }

   public String cardgram_next_1()
   {
      set("when", "next");
      return null;
   }

   public String cardgram_now_1()
   {
      set("when", "now");
      return null;
   }

   public String cardgram_NPD_1()
   {
      set("what", nodeValue());
      return null;
   }

   public String cardgram_NPD1_2()
   {
      String oldAggregation = (String) valueOf("aggregation");
      parser.raiseFeatures();
      if (oldAggregation != null) // e.g. "each" or "all"
      {  set("aggregation", oldAggregation);
         if (oldAggregation.equals("each"))
            // e.g. "each of the piles"
            set("number", "singular");
      }

      return null;
   }

   public String cardgram_NPL_1()
   {
      set("quantity", "several");
      set("type", nodeValue());
      set("number", "plural");
      return null;
   }

   public String cardgram_NPL_2()
   {
      set("quantity", "several");
      set("type", nodeValue());
      set("number", "plural");
      return null;
   }

   public String cardgram_NPL_3()
   {
      set("type", nodeValue());
      return null;
   }

   public String cardgram_NSING_1()
   {
      if (valueOf("quantity") == null)
         set("quantity", "one");
      set("number", "singular");
      set("type", (String) nodeValue());
      return null;
   }

   public String cardgram_NSING_2()
   {
      set("type", (String) nodeValue());
      return null;
   }

   public Object cardgram_pile_1_v()
   {
      return "pile";
   }

   public String cardgram_pile_2()
   {
      set("verb", "pile");
      set("command", "stack");
      return null;
   }

   public String cardgram_separately_1()
   {
      set("aggregation", "each");
      return null;
   }

   public String cardgram_shuffle_1()
   {
      set("verb", "shuffle");
      set("command", "shuffle");
      return null;
   }

   public String cardgram_spread_1()
   {
      set("verb", "spread");
      set("command", "spread");
      return null;
   }

   public String cardgram_stack_1()
   {
      set("verb", "stack");
      set("command", "stack");
      return null;
   }

   public String cardgram_the_1()
   {
      set("which", "quantified");
      return null;
   }

   public String cardgram_that_1()
   {
      set("which", "deictic"); // (or anaphoric without pointing)
      set("number", "singular");
      set("quantity", "one");
      return null;
   }

   public String cardgram_them_1()
   {
      set("which", "anaphoric");
      set("number", "plural");
      set("quantity", "several");
      return null;
   }

   public String cardgram_then_1()
   {
      set("when", "then");
      return null;
   }

   public String cardgram_there_1()
   {
      set("where", "there");
      return null;
   }

   public String cardgram_these_1()
   {
      set("which", "deictic");  // or definite without pointing
      set("number", "plural");
      set("quantity", "several");
      return null;
   }

   public String cardgram_this_1()
   {
      set("which", "deictic");  // or definite without pointing
      set("number", "singular");
      set("quantity", "one");
      return null;
   }

   public String cardgram_those_1()
   {
      set("which", "deictic");  // or anaphoric without pointing
      set("number", "plural");
      set("quantity", "several");
      return null;
   }

   public String cardgram_together_1()
   {
      set("aggregation", "together");
      return null;
   }

   public String cardgram_turn_1()
   {
      set("verb", "turn");
      set("command", "turn");
      return null;
   }

   public String cardgram_up_2()
   {
      set("orientation", "up");
      return null;
   }

   public String cardgram_VERB_1()
   {
      parser.raiseFeatures();
      return null;
   }

   // Instance variables:

   private ASDParser parser = null;

} // end class Cardgram1Semantics
